/*
 * complete channel data with single channel mode
 *  Copyright (C) 2017 Noriaki TANAKA (dtana@startide.jp)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 * $Id: cc.c,v 1.2 2017/01/07 10:32:55 dtana Exp $
 */
#include <stdio.h>
#include "dng.h"

static unsigned comp_ch(const struct dng *,
    int, int, enum DNG_CF, enum DNG_CF);

void
dng_comp_ch(struct dng *p, enum DNG_CF ch)
{
	int x, y, w, h;
	w = dng_w(p), h = dng_h(p);

	for (y = 0; y < h; y++)
		for (x = 0; x < w; x++) {
			enum DNG_CF dc;
			
			if ((dc = dng_cfac(p, x, y)) != ch)
				dng_put(p, x, y, comp_ch(p, x, y, ch, dc));
		}
}

#define BL(p,x,y) dng_black_level((p),(x),(y))

static unsigned
comp_ch(const struct dng *p,
    int cx, int cy, enum DNG_CF ch, enum DNG_CF dest_ch)
{
	int x, y, np, rp;

	np = 0, rp = 0;
	for (y = cy - 1; y <= cy + 1; y++)
		for (x = cx - 1; x <= cx + 1; x++) {
			int tp;

			if (dng_cfac(p, x, y) != ch)
				continue;
			if ((tp = dng_get(p, x, y)) < 0)
				continue;
			rp += (tp - BL(p, x, y)), np++;
		}
	rp /= np;
	if (ch == G)
		rp /= 2;
	if (dest_ch == G)
		rp *= 2;
	return (unsigned)(rp + BL(p, cx, cy));
}

/*

G B G B G B G B
R G R G R G R G
G B G B G B G B
R G R G R G R G
G B G B G B G B
R G R G R G R G

*/
