/*
 * color lookup table
 *  Copyright (C) 2015-2017 Noriaki TANAKA (dtana@startide.jp)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 * $Id: clut.c,v 1.3 2017/02/12 10:22:59 dtana Exp $
 */
#include <stdlib.h>
#include <stdio.h>
#include "dng.h"

/* correction lookup table */
struct dng_clut {
	unsigned r, g, b;
};

#define MAXCVAL 16384

struct dng_clut *
dng_clut_load(const char *path)
{
	struct dng_clut *p;
	FILE *fp;
	char b[BUFSIZ];
	int n;
	
	if ((p = calloc(sizeof(struct dng_clut), MAXCVAL)) == NULL)
		perror("malloc"), exit(1);
	if ((fp = fopen(path, "r")) == NULL)
		perror(path), exit(1);
	for (n = 0; fgets(b, BUFSIZ, fp) != NULL; n++) {
		if (n == MAXCVAL)
			fprintf(stderr,
			    "%s: correction LUT table too large\n", path),
			    exit(1);
		if (sscanf(b, "%u %u %u", &p[n].r, &p[n].g, &p[n].b) != 3)
			fprintf(stderr, "%s: LUT need RGB values\n", path),
			    exit(1);
	}
	if (ferror(fp))
		perror(path), exit(1);
	fclose(fp);
	return p;
}

void
dng_clut_apply(struct dng *p, const struct dng_clut *clut)
{
	int x, y, w, h;

	w = dng_w(p), h = dng_h(p);
	for (y = 0; y < h; y++)
		for (x = 0; x < w; x++) {
			unsigned c;
			
			c = (unsigned)dng_get(p, x, y);
			if (MAXCVAL <= c) {
				fprintf(stderr,
				    "%u: pixel value too large\n", c);
				c = MAXCVAL - 1;
			}
			switch (dng_cfac(p, x, y)) {
			case R:
				c = clut[c].r;
				break;
			case G:
				c = clut[c].g;
				break;
			case B:
				c = clut[c].b;
				break;
			}
			dng_put(p, x, y, c);
		}
}
