/*
 * dark frame processing
 *  Copyright (C) 2015-2017 Noriaki TANAKA (dtana@startide.jp)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 * $Id: dark.c,v 1.4 2017/02/12 10:22:59 dtana Exp $
 */
#include <stdio.h>
#include "dng.h"

/* mean to switch longtime exposure NR? */
#define CUSTOMRENDERED "CustomRendered"
#define BL(p,x,y) dng_black_level((p),(x),(y))

int
dng_dark(struct dng *base, const struct dng *dark)
{
	int x, y, w, h, bs;
	
	w = dng_w(base), h = dng_h(base);
	if (w != dng_w(dark) || h != dng_h(dark)) {
		fprintf(stderr, "size mismatch\n");
		return -1;
	}
	
	if ((bs = dng_estimated_bps(base) - dng_estimated_bps(dark)) != 0)
		fprintf(stderr,
		    "bitspersample mismatch, adjust and continue.\n");
	
	if (dng_metadata_get_intv(base, CUSTOMRENDERED) == 1)
		fprintf(stderr, "warning: base image applied "
		    "longtime exposure NR\n");
	if (dng_metadata_get_intv(dark, CUSTOMRENDERED) == 1)
		fprintf(stderr, "warning: dark image applied "
		    "longtime exposure NR\n");
	for (y = 0; y < h; y++)
		for (x = 0; x < w; x++) {
			unsigned bu, du;
			int bi;
			
			bu = (unsigned)dng_get(base, x, y);
			du = (unsigned)dng_get(dark, x, y);
			if (bs) {
				if (bs > 0)
					du <<= bs;
				else
					du >>= (-bs);
			}
			bi = (int)bu - (int)du + BL(base, x, y);
			if (bi < 0)
				bi = 0;
			dng_put(base, x, y, (unsigned)bi);
		}

	dng_metadata_set_intv(base, CUSTOMRENDERED, 1);
	
	return 0;
}
