/*
 * DNG processings
 *  Copyright (C) 2015 Noriaki TANAKA (dtana@startide.jp)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 * $Id: dng.h,v 1.28 2017/01/07 10:32:55 dtana Exp $
 */
#ifndef DNG_H
#define DNG_H

/* color code on CFA */
enum DNG_CF {
	R = 0,
	G = 1,
	B = 2
};

struct dng;
struct dng *load_dng(const char *);
void term_dng(struct dng *);
int save_dng(const struct dng *, const char *);
int dng_dark(struct dng *, const struct dng *),
	dng_flat(struct dng *, const struct dng *, const int [3]),
	dng_spot_fix(struct dng *);
void dng_comp_ch(struct dng *, enum DNG_CF);
int dng_spot_fix_with_dark(struct dng *, const struct dng *);
int dng_dump_spots(const struct dng *);

int dng_get(const struct dng *p, int x, int y);
void dng_put(struct dng *p, int x, int y, unsigned u);

int dng_metadata_set_intv(struct dng *, const char *, int);
int dng_metadata_get_intv(const struct dng *, const char *);
int dng_estimated_bps(const struct dng *);

struct dng_clut;
struct dng_clut *dng_clut_load(const char *);
void dng_clut_apply(struct dng *, const struct dng_clut *);

enum DNG_CF dng_cfac(const struct dng *p, int x, int y);
int dng_black_level(const struct dng *, int, int);
int dng_bps(const struct dng *);

struct dng_average;
struct dng_average *dng_average_init(const struct dng *);
void dng_average_term(struct dng_average *);
int dng_average_add(struct dng_average *, const struct dng *);
int dng_average_calc(const struct dng_average *, struct dng *);

int dng_w(const struct dng *), dng_h(const struct dng *);

#endif
