/*
 * flat frame processing
 *  Copyright (C) 2015-2017 Noriaki TANAKA (dtana@startide.jp)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the
 *  License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 * $Id: flat.c,v 1.7 2017/02/12 10:22:59 dtana Exp $
 */
#include <stdio.h>
#include "dng.h"

static double
calc_channel_ave(const struct dng *flat, enum DNG_CF ch)
{
	int x, y, w, h, n;
	double a;
	
	a = 0;
	n = 0;
	w = dng_w(flat), h = dng_h(flat);
	for (y = 0; y < h; y++)
		for (x = 0; x < w; x++)
			if (dng_cfac(flat, x, y) == ch)
				a += dng_get(flat, x, y), n++;
	return a / n;
}

#define BL(p,x,y) dng_black_level((p),(x),(y))

int
dng_flat(struct dng *base, const struct dng *flat, const int bias[3])
{
	int x, y, w, h;
	double a[3];
	
	w = dng_w(base), h = dng_h(base);
	if (w != dng_w(flat) || h != dng_h(flat)) {
		fprintf(stderr, "size mismatch\n");
		return -1;
	}
	a[R] = calc_channel_ave(flat, R);
	a[G] = calc_channel_ave(flat, G);
	a[B] = calc_channel_ave(flat, B);
	
	for (y = 0; y < h; y++)
		for (x = 0; x < w; x++) {
			int fx, bx, ax, ch, bs;
			
			ch = dng_cfac(flat, x, y);
			bs = bias[ch];
			fx = dng_get(flat, x, y) - BL(flat, x, y) + bs;
			if (fx < 1)
				fx = 0x0001;
			ax = a[ch] -                    BL(flat, x, y) + bs;
			if (ax < 1)
				ax = 0x0001;
			bx = dng_get(base, x, y) - BL(base, x, y) + bs;
			if (bx < 0)
				bx = 0x0000;
			bx = (bx * ax) / fx + BL(base, x, y);
			if (0xffff < bx)
				bx = 0xffff;
			dng_put(base, x, y, (unsigned)bx);
		}
	return 0;
}
